class Preset:
    DEFAULTS = {}

    def __init__(self, title, options):
        self.title = title
        self.options = options

    @property
    def markup(self):
        # print({self.__title: self.__options})
        return {self.title: self.options}

    def merge_defaults(self, params, **kwargs):
        return {**self.DEFAULTS, **params, **kwargs}

class ProjectPreset(Preset):
    def __init__(self, name, version, site_url, title='project', **kwargs):
        markup = {
            'name': name,
            'version': version,
            'site_url': site_url,
        }
        markup.update(**kwargs)
        super(ProjectPreset, self).__init__(title, markup)


class OptionsPreset(Preset):
    def __init__(self, debug, allowed_hosts, secret_key=None, title='options', **kwargs):
        markup = {
            'secret_key': secret_key,
            'debug': debug,
            'allowed_hosts': allowed_hosts,
            'database': None,
            'cache': None,
            'mailing': None,
        }
        markup.update(**kwargs)
        super(OptionsPreset, self).__init__(title, markup)


class DatabasePreset(Preset):
    def __init__(self, title, name, user, password, tcp_addr, tcp_port, **kwargs):
        markup = {
            'name': name,
            'user': user,
            'password': password,
            'tcp_addr': tcp_addr,
            'tcp_port': tcp_port,
        }
        markup.update(**kwargs)
        super(DatabasePreset, self).__init__(title, markup)


class CachePreset(Preset):
    def __init__(self, title, enabled, tcp_addr, tcp_port, **kwargs):
        markup = {
            'enabled': enabled,
            'tcp_addr': tcp_addr,
            'tcp_port': tcp_port
        }
        markup.update(**kwargs)
        super(CachePreset, self).__init__(title, markup)


class EmailPreset(Preset):
    def __init__(self, title, enabled, use_tls, host, host_user, port, default_from_email, feedback_email, **kwargs):
        markup = {
            'enabled': enabled,
            'use_tls': use_tls,
            'host': host,
            'host_user': host_user,
            'port': port,
            'default_from_email': default_from_email,
            'feedback_email': feedback_email,
        }
        markup.update(**kwargs)
        super(EmailPreset, self).__init__(title, markup)





from ..generic import EmailPreset


class MailDevPreset(EmailPreset):
    DEFAULTS = {
        'title': 'maildev',
        'use_tls': False,
        'host': '127.0.0.1',
        'host_user': None,
        'port': 1025,
    }

    def __init__(self, enabled, default_from_email=None, feedback_email=None, **kwargs):
        params = {
            'enabled': enabled,
            'default_from_email': default_from_email,
            'feedback_email': feedback_email,
        }
        params = self.merge_defaults(params, **kwargs)
        super(MailDevPreset, self).__init__(**params)



from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from confidence import Configuration


class Command(BaseCommand):
    help = 'Checks if project configuration file already exists.'

    def handle(self, *args, **options):
        project_conf_variable = 'PROJECT_CONF'
        if not hasattr(settings, project_conf_variable):
            CommandError('Seems like there\'s no {} variable specified in your settings.py. '
                         'You need to set it with a Configuration instance first '
                         'to use this management command.'.format(project_conf_variable))

        project_conf = getattr(settings, project_conf_variable)
        if not isinstance(project_conf, Configuration):
            CommandError('Variable {} in settings.py must be a Configuration instance.'.format(project_conf_variable))

        if not project_conf.exists():
            message = self.style.WARNING('[WARNING] Configuration file doesn\'t '
                                         'exist at {}.'.format(project_conf.filepath))
        else:
            message = self.style.SUCCESS('[SUCCESS] Configuration file found at {}'.format(project_conf.filepath))
        self.stdout.write(message)



from ..generic import DatabasePreset


class MySQLPreset(DatabasePreset):
    DEFAULTS = {
        'title': 'mysql',
        'tcp_addr': '127.0.0.1',
        'tcp_port': 3306,
    }

    def __init__(self, name, user, password, **kwargs):
        params = {
            'name': name,
            'user': user,
            'password': password,
        }
        params = self.merge_defaults(params, **kwargs)
        super(MySQLPreset, self).__init__(**params)


class PostgreSQLPreset(DatabasePreset):
    DEFAULTS = {
        'title': 'postgresql',
        'tcp_addr': '127.0.0.1',
        'tcp_port': 5432,
    }

    def __init__(self, name, user, password, **kwargs):
        params = {
            'name': name,
            'user': user,
            'password': password,
        }
        params = self.merge_defaults(params, **kwargs)
        super(PostgreSQLPreset, self).__init__(**params)




from ..generic import CachePreset


class RedisPreset(CachePreset):
    DEFAULTS = {
        'title': 'redis',
        'tcp_addr': 'redis://127.0.0.1',
        'tcp_port': 6379,
    }

    def __init__(self, enabled, **kwargs):
        params = {
            'enabled': enabled,
        }
        params = self.merge_defaults(params, **kwargs)
        super(RedisPreset, self).__init__(**params)



from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from confidence import Configuration


class Command(BaseCommand):
    help = 'Creates project configuration file.'

    def add_arguments(self, parser):
        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            help='Force management command to rewrite configuration file with defaults if it already exists.',
        )

    def handle(self, *args, **options):
        project_conf_variable = 'PROJECT_CONF'
        if not hasattr(settings, project_conf_variable):
            CommandError('Seems like there\'s no {} variable specified in your settings.py. '
                         'You need to set it with a Configuration instance first '
                         'to use this management command.'.format(project_conf_variable))

        project_conf = getattr(settings, project_conf_variable)
        if not isinstance(project_conf, Configuration):
            CommandError('Variable {} in settings.py must be a Configuration instance.'.format(project_conf_variable))

        try:
            project_conf.make(force=options.get('force'))
        except (FileExistsError, OSError) as ex:
            message = self.style.ERROR('[ERROR] Can\'t create a configuration file at {}.'
                                       '\nREASON: {}'.format(project_conf.filepath, ex))
        else:
            message = self.style.SUCCESS('[SUCCESS] Created a configuration file at {}'.format(project_conf.filepath))

        self.stdout.write(message)



import configparser
import os


class Configuration:
    @classmethod
    def compile_from_presets(cls, filepath, presets, instant_start=False):
        markup = {}
        for preset in presets:
            markup.update(preset.markup)
        return cls(filepath, markup, instant_start)

    def __init__(self, filepath, markup, instant_start=False):
        self.filepath = filepath
        self.markup = markup
        self.instant_start = instant_start

        if not self.exists():
            self.make()

    def exists(self):
        return os.path.exists(self.filepath)

    def make(self, force=False):
        if self.exists() and not force:
            raise FileExistsError('File already exists.')

        directory, filename = os.path.split(self.filepath)
        if directory and not os.path.exists(directory):
            os.makedirs(directory)

        conf = configparser.ConfigParser()

        for section, option_dict in self.markup.items():
            conf[section] = {}
            for option, value_raw in option_dict.items():
                if value_raw is None:
                    value = ''
                elif isinstance(value_raw, list):
                    value = ', '.join([str(e) for e in value_raw])
                else:
                    value = str(value_raw)

                conf[section][option] = value

        with open(self.filepath, 'w') as f:
            conf.write(f)

        if not self.instant_start:
            print('Configuration file created at {}. You may want to edit it before application start.'.format(self.filepath))
            exit()

    def get(self, section, option):
        config = configparser.ConfigParser()
        config.read(self.filepath)

        try:
            return config[section][option]
        except (configparser.NoSectionError, configparser.NoOptionError, KeyError):
            markup_options = self.markup.get(section)
            if not markup_options:
                return
            markup_value = markup_options.get(option)
            return markup_value

    def get_bool(self, section, option):
        value = self.get(section, option)
        value_map = {'True': True, 'False': False}

        result = value_map.get(value)
        if result:
            return result
        return bool(result)

    def get_csv(self, section, option):
        value = self.get(section, option)
        return value.replace(' ', '').split(',')

    def get_int(self, section, option):
        value = self.get(section, option)
        return int(value)

    def get_float(self, section, option):
        value = self.get(section, option)
        return float(value)



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-confidence',
    version='0.1.0',
    packages=find_packages(),
    include_package_data=True,
    license='Apache 2.0 License',
    description='A Django app to manage project configuration files.',
    long_description=README,
    url='https://volynov.ru/',
    author='Mikhail volynov',
    author_email='mikhail@volynov.ru',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
